/**
 * @file
 * Provides mobile toggler for the the Ultimenu main block.
 */

(function ($, Drupal, drupalSettings, window, document) {

  'use strict';

  var _hidingTimer = void 0;
  var _isExpanded = 'is-ultimenu-expanded';
  var _isHiding = 'is-ultimenu-canvas--hiding';
  var _ultimenuMain = '[data-ultimenu="main"]';
  var $body = $('body');
  var $hamburger = $('[data-ultimenu-button]');

  Drupal.ultimenu = Drupal.ultimenu || {
    documentWidth: 0,
    doResizeMain: function () {
      var me = this;
      var width = $(document).width();

      // Do not cache the selector, to avoid incorrect classes with its cache.
      if ($('[data-ultimenu-button]').is(':hidden')) {
        $body.removeClass(_isExpanded);
      }

      if (width !== me.documentWidth) {
        return;
      }

      if ($('[data-ultimenu-button]').is(':hidden')) {
        $body.removeClass(_isExpanded);
      }
      else if (width !== me.documentWidth) {
        $body.addClass(_isExpanded);
      }

      me.documentWidth = width;
    },

    triggerAjax: function (e) {
      var $trigger = $(e.target);
      var $ajax = $trigger.closest('.has-ultimenu').find('.ultiajax__fallback');
      if ($ajax.length && !$ajax.attr('ultiajaxHit')) {
        $ajax.click();

        // Removes attribute to prevent the event from firing again.
        $trigger.removeAttr('data-ultiajax-trigger');
        $ajax.attr('data-ultiajax-hit', 1);
        e.preventDefault();
      }
    },

    triggerClickHamburger: function (e) {
      if (e.target === this) {
        $hamburger.trigger('click');
      }
    },

    doClickHamburger: function (e) {
      e.preventDefault();

      var $button = $(this);

      $body.toggleClass(_isExpanded);
      $button.toggleClass('is-ultimenu-button-active');

      $('.is-ultimenu-item-expanded', _ultimenuMain).removeClass('is-ultimenu-item-expanded');
      $(_ultimenuMain + ' .ultimenu__link')
        .removeClass('is-ultimenu-active')
        .next('.ultimenu__flyout')
        .slideUp();

      if (!$body.hasClass(_isExpanded)) {
        window.clearTimeout(_hidingTimer);
        $body.addClass(_isHiding);
        _hidingTimer = window.setTimeout(function () {
          $body.removeClass(_isHiding);
        }, 600);
      }
    },

    doClickCaret: function (e) {
      e.preventDefault();

      var $caret = $(this);

      $caret.closest('li').toggleClass('is-ultimenu-item-expanded');
      $caret.parent().toggleClass('is-ultimenu-active');
      $caret.parent().next('.ultimenu__flyout')
        .not(':animated').slideToggle();
    },

    onResize: function (c, t) {
      window.onresize = function () {
        window.clearTimeout(t);
        t = window.setTimeout(c, 200);
      };
      return c;
    }

  };

  /**
   * Ultimenu utility functions for the main menu only.
   *
   * @param {int} i
   *   The index of the current element.
   * @param {HTMLElement} elm
   *   The ultimenu HTML element.
   */
  function doUltimenu(i, elm) {
    var me = Drupal.ultimenu;
    var $elm = $(elm);
    var settings = drupalSettings.ultimenu || {};
    var $link = $elm.find('.ultimenu__link');
    var $backdrop = $('.is-ultimenu-canvas-backdrop');
    var $offCanvas = $('.is-ultimenu-canvas-off');

    // Applies to the main Ultimenu.
    if ($elm.data('ultimenu') === 'main') {
      if (settings && (settings.canvasOff && settings.canvasOn)) {
        if (!$offCanvas.length) {
          $(settings.canvasOff).addClass('is-ultimenu-canvas-off');
        }

        $(settings.canvasOn).addClass('is-ultimenu-canvas-on');
      }

      $offCanvas = $('.is-ultimenu-canvas-off');
      if (!$backdrop.length && $offCanvas.length) {
        $offCanvas.before('<div class="is-ultimenu-canvas-backdrop" />');
      }
    }

    // Applies to other Ultimenus.
    $('.caret', elm).once('ultimenu-caret').click(me.doClickCaret);
    $body.off().on('click.ultimenuBackdrop', '.is-ultimenu-canvas-backdrop', me.triggerClickHamburger);
  }

  /**
   * Attaches Ultimenu behavior to HTML element [data-ultimenu].
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.ultimenu = {
    attach: function (context) {
      var me = Drupal.ultimenu;

      // Modifies functionality for the Ultimenu menus.
      $('[data-ultimenu]', context).once('ultimenu').each(doUltimenu);

      // Reacts on clicking Ultimenu hamburger button.
      $hamburger.once('ultimenu-button').click(me.doClickHamburger);

      // Reacts on resizing Ultimenu.
      me.onResize(me.doResizeMain.bind(me))();
    }
  };

})(jQuery, Drupal, drupalSettings, this, this.document);
